import { exportDataGrid as exportToExcel } from 'devextreme/excel_exporter'
import { saveAs } from 'file-saver'
import { jsPDF } from 'jspdf'
import autoTable from 'jspdf-autotable'
import { Workbook } from 'exceljs'
import { setHeaderStyle } from '@/report/utils/xlsx'
import { formatNumber, formatPercentage } from '@/utils/numbers'
import { exportToWord } from './doc/Anomali_LAPPGP_LPT'

const reportName = 'Anomali Penanganan Pengaduan Gangguan Petugas'
const fontSize = 5

const groupingData = (data: any, reportMeta: any) => {
  const groupedData: any = {}

  const reportMetaMapping: any = {
    up3: 'nama_ulp',
    uid: 'nama_up3',
    default: 'nama_uid'
  }

  let key = 'nama_uid'
  for (const prop in reportMetaMapping) {
    if (reportMeta[prop] && reportMeta[prop].id != 0) {
      key = reportMetaMapping[prop]
      break
    }
  }

  data.forEach((item: any) => {
    const groupKey = item[key]

    if (!groupedData[groupKey]) {
      groupedData[groupKey] = { data: [] }
    }

    groupedData[groupKey].data.push(item)
  })

  for (const key in groupedData) {
    const data = groupedData[key].data

    data.forEach((item: any) => {
      if (!groupedData[key].summary) {
        groupedData[key].summary = {
          total_petugas: 0,
          petugas_pln_mobile_marking: 0,
          petugas_pln_mobile_nonmarking: 0,
          petugas_cc123_marking: 0,
          petugas_cc123_nonmarking: 0
          // petugas_anomali_marking: 0,
          // petugas_anomali_nonmarking: 0,
          // persen_pln_mobile_marking
          // persen_pln_mobile_non_marking
          // persen_cc123_marking
          // persen_cc123_nonmarking
          // persen_anomali_marking
          // persen_anomali_non_marking
        }
      }

      groupedData[key].summary.total_petugas += item.total_petugas
      groupedData[key].summary.petugas_pln_mobile_marking += item.petugas_pln_mobile_marking
      groupedData[key].summary.petugas_pln_mobile_nonmarking += item.petugas_pln_mobile_nonmarking
      groupedData[key].summary.petugas_cc123_marking += item.petugas_cc123_marking
      groupedData[key].summary.petugas_cc123_nonmarking += item.petugas_cc123_nonmarking
    })

    groupedData[key].summary.petugas_anomali_marking =
      groupedData[key].summary.petugas_pln_mobile_marking +
      groupedData[key].summary.petugas_cc123_marking
    groupedData[key].summary.petugas_anomali_nonmarking =
      groupedData[key].summary.petugas_pln_mobile_nonmarking +
      groupedData[key].summary.petugas_cc123_nonmarking

    groupedData[key].summary.persen_pln_mobile_marking =
      (groupedData[key].summary.petugas_pln_mobile_marking /
        groupedData[key].summary.total_petugas) *
      100

    groupedData[key].summary.persen_pln_mobile_non_marking =
      (groupedData[key].summary.petugas_pln_mobile_nonmarking /
        groupedData[key].summary.total_petugas) *
      100

    groupedData[key].summary.persen_cc123_marking =
      (groupedData[key].summary.petugas_cc123_marking / groupedData[key].summary.total_petugas) *
      100

    groupedData[key].summary.persen_cc123_nonmarking =
      (groupedData[key].summary.petugas_cc123_nonmarking / groupedData[key].summary.total_petugas) *
      100

    groupedData[key].summary.persen_anomali_marking =
      groupedData[key].summary.persen_pln_mobile_marking +
      groupedData[key].summary.persen_cc123_marking
    groupedData[key].summary.persen_anomali_non_marking =
      groupedData[key].summary.persen_pln_mobile_non_marking +
      groupedData[key].summary.persen_cc123_nonmarking
  }

  return groupedData
}

const getTitle = (reportMeta: any) => {
  if (reportMeta.up3.id != 0) {
    return 'nama_up3'
  }

  if (reportMeta.uid.id != 0) {
    return 'nama_uid'
  }

  return 'nama_regional'
}

const formatData = (rawData: any, reportMeta: any) => {
  const formattedData: any = []
  const groupedData: any = {}

  const data = groupingData(rawData, reportMeta)
  const title = getTitle(reportMeta)

  for (const key in data) {
    const groupKey = data[key].data[0][title]

    if (!groupedData[groupKey]) {
      groupedData[groupKey] = { data: [] }
    }

    groupedData[groupKey].data.push({
      key: key,
      value: data[key]
    })
  }

  for (const key in groupedData) {
    formattedData.push([{ content: key, colSpan: 14, styles: { fontStyle: 'bold' } }])

    groupedData[key].data.forEach((item: any) => {
      const summary = item.value.summary

      formattedData.push([
        item.key,
        formatNumber(summary.total_petugas),
        formatNumber(summary.petugas_pln_mobile_marking),
        formatNumber(summary.petugas_pln_mobile_nonmarking),
        formatNumber(summary.petugas_cc123_marking),
        formatNumber(summary.petugas_cc123_nonmarking),
        formatNumber(summary.petugas_anomali_marking),
        formatNumber(summary.petugas_anomali_nonmarking),
        formatPercentage(summary.persen_pln_mobile_marking),
        formatPercentage(summary.persen_pln_mobile_non_marking),
        formatPercentage(summary.persen_cc123_marking),
        formatPercentage(summary.persen_cc123_nonmarking),
        formatPercentage(summary.persen_anomali_marking),
        formatPercentage(summary.persen_anomali_non_marking)
      ])
    })
  }

  return formattedData
}

const formatMetaData = (reportMeta: any) => {
  const periode = reportMeta.periode ? reportMeta.periode.split(' s/d ') : ''

  let dateFromFormat = ''
  let dateToFormat = ''
  let dayTo = ''

  if (periode != '') {
    const dateFrom = new Date(periode[0].split('-').reverse().join('-'))
    const dateTo = new Date(periode[1].split('-').reverse().join('-'))

    dateFromFormat = `${dateFrom.getDate()}-${dateFrom.toLocaleString('id-ID', {
      month: 'long'
    })}-${dateFrom.getFullYear()}`

    dateToFormat = `${dateTo.getDate()}-${dateTo.toLocaleString('id-ID', {
      month: 'long'
    })}-${dateTo.getFullYear()}`

    dayTo = dateTo.toLocaleString('id-ID', { weekday: 'long' })
  }

  return { dateFromFormat, dateToFormat, dayTo }
}

const exportToPDF = (reportMeta: any, rawData: any, preview: boolean = false) => {
  const day = new Date().toLocaleString('id-ID', { weekday: 'long' })
  const date = new Date().getDate()
  const month = new Date().toLocaleString('id-ID', { month: 'long' })
  const year = new Date().getFullYear()
  const data = formatData(rawData, reportMeta)
  const meta = formatMetaData(reportMeta)
  const doc = new jsPDF({
    orientation: 'landscape'
  })

  autoTable(doc, {
    head: [
      ['PT. PLN(Persero)', '', ''],
      [
        { content: 'UNIT INDUK', styles: { cellWidth: 40 } },
        { content: ':', styles: { cellWidth: 1 } },
        reportMeta.uid
          ? reportMeta.uid.name.toUpperCase()
          : 'Semua Unit Induk Distribusi/Wilayah'.toUpperCase()
      ],
      [
        'UNIT PELAKSANA PELAYANAN PELANGGAN',
        ':',
        reportMeta.up3
          ? reportMeta.up3.name.toUpperCase()
          : 'Semua Unit Pelaksanaan Pelayanan Pelanggan'.toUpperCase()
      ],
      [
        'UNIT LAYANAN PELANGGAN',
        ':',
        reportMeta.ulp
          ? reportMeta.ulp.name.toUpperCase()
          : 'Semua Unit Layanan Pelanggan'.toUpperCase()
      ],
      ['JENIS LAPORAN', ':', reportMeta.jenisLaporan.name.toUpperCase()]
    ],
    styles: {
      fontSize,
      cellPadding: 0.1,
      textColor: [0, 0, 0],
      fontStyle: 'bold'
    },
    theme: 'plain',
    startY: 10
  })

  autoTable(doc, {
    head: [
      [`${reportName}`.toUpperCase()],
      [`PERIODE TANGGAL : ${meta.dateFromFormat} SD TGL ${meta.dateToFormat}`]
    ],
    styles: {
      fontSize,
      cellPadding: 0.1,
      textColor: [0, 0, 0],
      fontStyle: 'bold',
      halign: 'center'
    },
    theme: 'plain',
    startY: 25
  })

  autoTable(doc, {
    head: [
      [
        {
          content: 'Nama Unit',
          rowSpan: 4
        },
        'Total Petugas',
        {
          content: 'Total Petugas Yang Pengaduan Diselesaikan Secara Anomali',
          colSpan: 6
        },
        {
          content: '% Petugas Yang Pengaduan Diselesaikan Secara Anomali',
          colSpan: 6
        }
      ],
      [
        {
          content: 'a',
          rowSpan: 3
        },
        {
          content: 'PLN Mobile',
          colSpan: 2
        },
        {
          content: 'CC 123',
          colSpan: 2
        },
        {
          content: 'Total',
          colSpan: 2
        },
        {
          content: 'PLN Mobile',
          colSpan: 2
        },
        {
          content: 'CC 123',
          colSpan: 2
        },
        {
          content: 'Total',
          colSpan: 2
        }
      ],
      [
        {
          content: 'b',
          colSpan: 2
        },
        {
          content: 'c',
          colSpan: 2
        },
        {
          content: 'd=b+c',
          colSpan: 2
        },
        {
          content: 'e=b/a',
          colSpan: 2
        },
        {
          content: 'f=c/a',
          colSpan: 2
        },
        {
          content: 'g=e+f',
          colSpan: 2
        }
      ],
      [
        'Marking',
        'Non Marking',
        'Marking',
        'Non Marking',
        'Marking',
        'Non Marking',
        'Marking',
        'Non Marking',
        'Marking',
        'Non Marking',
        'Marking',
        'Non Marking'
      ]
    ],
    body: data,
    styles: {
      fontSize,
      cellPadding: 1,
      lineColor: [0, 0, 0],
      lineWidth: 0.1,
      cellWidth: 'auto'
    },
    rowPageBreak: 'auto',
    headStyles: {
      fillColor: [192, 192, 192],
      textColor: [0, 0, 0],
      fontStyle: 'bold',
      halign: 'center',
      valign: 'middle'
    },
    bodyStyles: {
      textColor: [0, 0, 0]
    },
    didParseCell: function (data) {
      if (data.row.section === 'head') {
        data.cell.text = data.cell.text.map(function (word: any) {
          return word.toUpperCase()
        })
      }

      if (data.cell.text[0] === 'TOTAL') {
        for (const key in data.row.cells) {
          data.row.cells[key].styles.fillColor = [192, 192, 192]
          data.row.cells[key].styles.fontStyle = 'bold'
        }
      }
    },
    startY: 35
  })

  autoTable(doc, {
    head: [
      [`${day}, ${date}-${month}-${year}`],
      [
        {
          content: '(.........................................)',
          styles: { minCellHeight: 8, valign: 'bottom' }
        }
      ]
    ],
    styles: {
      fontSize,
      cellPadding: 0.1,
      textColor: [0, 0, 0],
      fontStyle: 'bold',
      halign: 'center'
    },
    theme: 'plain',
    tableWidth: 50,
    margin: { left: 230 }
  })

  if (preview) {
    doc.setProperties({
      title: `${reportName}`
    })
    window.open(doc.output('bloburl'))
  } else {
    doc.save(`Laporan ${reportName}.pdf`, { returnPromise: true }).then(() => {
      console.log('pdf berhasil disimpan')
    })
  }
}

const exportToXLSX = (reportMeta: any, e: any) => {
  const meta = formatMetaData(reportMeta)
  const workbook = new Workbook()
  const worksheet = workbook.addWorksheet(`${reportName}`)

  setHeaderStyle(worksheet, 1, 1, 'PT. PLN(Persero)')
  setHeaderStyle(
    worksheet,
    2,
    1,
    `UNIT INDUK : ${
      reportMeta.uid
        ? reportMeta.uid.name.toUpperCase()
        : 'Semua Unit Induk Distribusi/Wilayah'.toUpperCase()
    }`
  )
  setHeaderStyle(
    worksheet,
    3,
    1,
    `UNIT PELAKSANA PELAYANAN PELANGGAN : ${
      reportMeta.up3
        ? reportMeta.up3.name.toUpperCase()
        : 'Semua Unit Pelaksanaan Pelayanan Pelanggan'.toUpperCase()
    }`
  )
  setHeaderStyle(
    worksheet,
    4,
    1,
    `UNIT LAYANAN PELANGGAN : ${
      reportMeta.ulp
        ? reportMeta.ulp.name.toUpperCase()
        : 'Semua Unit Layanan Pelanggan'.toUpperCase()
    }`
  )
  setHeaderStyle(worksheet, 5, 1, `JENIS LAPORAN : ${reportMeta.jenisLaporan.name.toUpperCase()}`)

  setHeaderStyle(worksheet, 7, 1, `${reportName}`.toUpperCase(), true)
  setHeaderStyle(
    worksheet,
    8,
    1,
    `PERIODE TANGGAL : ${meta.dateFromFormat} SD TGL ${meta.dateToFormat}`,
    true
  )

  worksheet.mergeCells('A1:N1')
  worksheet.mergeCells('A2:N2')
  worksheet.mergeCells('A3:N3')
  worksheet.mergeCells('A4:N4')
  worksheet.mergeCells('A5:N5')
  worksheet.mergeCells('A7:N7')
  worksheet.mergeCells('A8:N8')

  exportToExcel({
    component: e.component,
    worksheet,
    autoFilterEnabled: true,
    topLeftCell: { row: 10, column: 1 },
    loadPanel: {
      enabled: false
    }
  }).then(() => {
    workbook.xlsx.writeBuffer().then((buffer: any) => {
      saveAs(new Blob([buffer], { type: 'application/octet-stream' }), `Laporan ${reportName}.xlsx`)
    })
  })

  e.cancel = true
}

const exportToDOCX = (reportMeta: any, rawData: any) => {
  const meta = formatMetaData(reportMeta)
  exportToWord(reportMeta, meta, formatData(rawData, reportMeta), reportName)
}

export { exportToPDF, exportToXLSX, exportToDOCX }
